////////////////////////////////////////////////////////////
//                 ,                 //
//                                    //
//                        v 1.2                           //
//                                                        //
//              Copyright (C) 2007 Sagrer                 //
//             LGPL 2.1         //
//                  .  lgpl.txt                     //
//                                                        //
//                  sagrer@yandex.ru                      //
////////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

unit KOLFileInString;

interface

uses
  Windows, messages, KOL;

type
  TFileInString = Class (TObject)
    //       . +   .
    FileString : ANSIString;
    FirstLoaded : boolean;
    Constructor Init;
    Function Siz : int64;
    Function Load(const FName : string) : boolean;
    Procedure Save(const FName : string);
    Procedure MakNewFile;
    Procedure AddChar(const ch : char);
    Procedure AddByte(const bt : byte);
    Function ReadStr(const FirstByte, BytesRead : int64) : string;
    Procedure LoadFromResource(const ResName, ResType : PChar);
  end;  

implementation

const
  fmOpenRead       = $0000;
  fmOpenWrite      = $0001;
  fmOpenReadWrite  = $0002;
  fmShareExclusive = $0010;
  fmShareDenyWrite = $0020;
  fmShareDenyNone  = $0040;

Constructor TFileInString.Init;
begin
  //.   .
  FirstLoaded := false;
  FileString := '';
  SetLength(FileString,Siz);
end;

function FileOpen(const FileName: string; Mode: LongWord): Integer;
//,   SysUtils.   \  
const
  AccessMode: array[0..2] of LongWord = (
    GENERIC_READ,
    GENERIC_WRITE,
    GENERIC_READ or GENERIC_WRITE);
  ShareMode: array[0..4] of LongWord = (
    0,
    0,
    FILE_SHARE_READ,
    FILE_SHARE_WRITE,
    FILE_SHARE_READ or FILE_SHARE_WRITE);
begin
  Result := -1;
  if ((Mode and 3) <= fmOpenReadWrite) and
    (((Mode and $F0) shr 4) <= fmShareDenyNone) then
    Result := Integer(CreateFile(PChar(FileName), AccessMode[Mode and 3],
      ShareMode[(Mode and $F0) shr 4], nil, OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL, 0));
end;

Function TFileInString.Load(const FName : string) : boolean;
{var
  I : int64;
  F1 : file;
  //f2 : file of byte;
  buf : array[1..2048] of char;
  NumRead : integer;
  OldFilMod : byte;

begin
  OldFilMod := FileMode;
  FileMode := 0;  // 
  Result := false;
  //   .
  //     .
  //      :).
  If FileExists(FName) = true then begin
    //      ....
    // 
    AssignFile(F1,FName);
    Reset(F1,1);
    // 
    SetLength(FileString,FileSize(FName));
    FileString := '';
    // ...
      repeat
      BlockRead(F1,buf,sizeof(buf),numread);
      //  
      if NumRead = SizeOf(Buf) then begin
        FileString := FileString+buf
      end
      //  
      else if numread <> 0 then begin
        I := 0;
        repeat
          I := I+1;
          FileString := FileString+buf[I];
        until I = numread;
        //siz := siz+numread;
      end;
    until NumRead <> SizeOf(Buf);
    //    ,      .
    CloseFile(F1);
    FirstLoaded := true;
    Result := true;
  end
  else begin
    //   ( ),   !
    MsgOK('!     , ..   .  .');
  end;
  FileMode := OldFilMod;   // ...
end;}

//  ...

var
  DataFile: THandle;
  FileSize: DWORD;
  Resultt : integer;
begin
  Result := false;
  FileString := '';
  //   .
  //     .
  //      :).
  If FileExists(FName) = true then begin
    DataFile := FileOpen(FName, fmOpenRead or fmShareDenyNone);
    if DataFile = 0 then
      MessageBox(Applet.Handle,PChar('Error, loading'+FName),'Error',MB_OK or MB_ICONERROR);
    try
      FileSize := GetFileSize(DataFile, nil);
      SetLength(FileString, FileSize);
      ReadFile(THandle(DataFile),Pointer(FileString)^,FileSize,LongWord(Resultt),nil)
    finally
      CloseHandle(DataFile);
    end;
    FirstLoaded := true;
    Result := true;
  end
  else begin
    //   ( ),   !
    MsgOK('!     , ..   .  .');
  end;
end;


procedure TFileInString.Save(const FName : string);
{var
  F1 : file;
  F : integer;
  buf : array[1..2048] of char;
  I, J : int64;
begin
  // .
  If Siz <> 0 then begin
    if FirstLoaded = true then begin
      //   ,  .
      //      .
      AssignFile(F1,FName);
      Rewrite(f1,1);
      J := Siz;
      F:= 0;
      //  .
      repeat
        I := 0;
        Repeat
          //    .
          I := I+1;
          //F -  ""    .
          F := F+1;
          //J -   -       .
          J := J-1;
          buf[I] := FileString[F];
        until (I = SizeOf(Buf)) or (J = 0);
        BlockWrite(F1,Buf,I);
      until J = 0;
      CloseFile(F1);
    end
    else begin
      //  ,   !
      MsgOK(',          !     ???  ,    :(');
    end;
  end
  else begin
    //   (0 )
    AssignFile(F1,FName);
    Rewrite(f1,1);
    CloseFile(F1);
  end;
end;}

var
  DataFile: THandle;
  FileSize: DWORD;
  Resultt : integer;

begin
  //   .
  //     .
  //      :).
  If Siz <> 0 then begin
    if FirstLoaded = true then begin
      //   ,  .
      if DirectoryExists(ExtractFilePath(FName)) = false then begin
        //    ,    ...
        ForceDirectories(ExtractFilePath(FName));
      end;
      //    -     .
      DataFile := CreateFile(PChar(FName),GENERIC_READ or GENERIC_WRITE,0,nil,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
      //DataFile := FileOpen(FName, fmOpenWrite or fmShareDenyWrite);
      if DataFile = 0 then
        MessageBox(Applet.Handle,PChar('Error, Saving'+FName),'Error',MB_OK or MB_ICONERROR);
      try
        FileSize := siz;
        //SetLength(FileString, FileSize);
        WriteFile(THandle(DataFile),Pointer(FileString)^,FileSize,LongWord(Resultt),nil)
      finally
        CloseHandle(DataFile);
      end;
      FirstLoaded := true;
    end
    else begin
      //  ,   !
      MsgOK(',          !     ???  ,    :(');
    end;
  end
  else begin
    //   (0 )
    //AssignFile(F1,FName);
    //Rewrite(f1,1);
    //CloseFile(F1);

    //try
      DataFile := CreateFile(PChar(FName),GENERIC_READ or GENERIC_WRITE,0,nil,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
    //finally
      CloseHandle(DataFile);
    //end;
  end;
end;

Procedure TFileInString.MakNewFile;
begin
  //  ""  -     .
  FirstLoaded := true;
  //SetLength(FileString,Siz);
  FileString := '';
end;

Procedure TFileInString.AddChar(const ch : char);
begin
  //       .
  FileString := FileString+ch;
end;

Procedure TFileInString.AddByte(const bt : byte);
begin
  //       ...
  FileString := FileString+char(bt);
end;

Function TFileInString.Siz : int64;
begin
  //    ...
  Result := Length(FileString);
end;

Function TFileInString.ReadStr(const FirstByte, BytesRead : int64) : string;
var
  TempInt, LastByte : int64;
begin
  //     .
  //   .
  TempInt := Length(FileString);
  LastByte := FirstByte+BytesRead;

  If LastByte <= TempInt+1 then begin
    //  LastByte     .
    //(    -    ,    )
    //     :).
    Result := CopyEnd(FileString,FirstByte);
    SetLength(Result,LastByte-FirstByte);
  end
  else begin
    result := '';
  end;  
end;

Procedure TFileInString.LoadFromResource(const ResName, ResType : PChar);
var
  MemStream : PStream;
  Buffer : array of byte;
begin
  //       ,   .
  MemStream := NewMemoryStream;   //   
  Resource2Stream(MemStream,HInstance,ResName,ResType);  //     
  MemStream.Position := 0;  // -   .
  SetLength(Buffer,MemStream.Size);   //  ( ).
  MemStream.Read(Buffer[0],MemStream.Size);  //    .
  FileString := string(Buffer);  //  -  .
  MemStream.Free;  // .
end;

end.
